@@ -150,7 +150,9 @@ size_t          length_String       (const iString *);
 size_t          size_String         (const iString *);
 iString *       mid_String          (const iString *, size_t charStartPos, size_t charCount);
 iString *       upper_String        (const iString *);
+iString *       upperLang_String    (const iString *, const char *langCode);
 iString *       lower_String        (const iString *);
+iString *       lowerLang_String    (const iString *, const char *langCode);
 iStringList *   split_String        (const iString *, const char *separator);
 iChar           first_String        (const iString *);
 iChar           last_String         (const iString *);
