@@ -39,9 +39,7 @@ struct Impl_RecentUrl {
     float        normScrollY;    /* normalized to document height */
     iGmResponse *cachedResponse; /* kept in memory for quicker back navigation */
     iGmDocument *cachedDoc;      /* cached copy of the presentation: layout and media (not serialized) */
-    struct {
-        uint8_t openedFromSidebar : 1;
-    } flags;
+    uint16_t     flags;
 };
 
 iDeclareType(MemInfo)
@@ -58,19 +56,21 @@ iDeclareTypeConstruction(History)
 iDeclareTypeSerialization(History)
 
 iHistory *  copy_History                (const iHistory *);
+void        lock_History                (iHistory *);
+void        unlock_History              (iHistory *);
 
 void        clear_History               (iHistory *);
 void        add_History                 (iHistory *, const iString *url);
+void        undo_History                (iHistory *); /* removes the most recent URL */
 void        replace_History             (iHistory *, const iString *url);
 void        setCachedResponse_History   (iHistory *, const iGmResponse *response);
-void        setCachedDocument_History   (iHistory *, iGmDocument *doc, iBool openedFromSidebar);
+void        setCachedDocument_History   (iHistory *, iGmDocument *doc);
 iBool       goBack_History              (iHistory *);
 iBool       goForward_History           (iHistory *);
-iBool       preceding_History           (iHistory *d, iRecentUrl *recent_out);
-//iBool       following_History           (iHistory *d, iRecentUrl *recent_out);
+iRecentUrl *precedingLocked_History     (iHistory *); /* requires manual lock/unlock! */
 iRecentUrl *recentUrl_History           (iHistory *, size_t pos);
 iRecentUrl *mostRecentUrl_History       (iHistory *);
-iRecentUrl *findUrl_History             (iHistory *, const iString *url);
+//iRecentUrl *findUrl_History             (iHistory *, const iString *url, int timeDir);
 
 void        clearCache_History                  (iHistory *);
 size_t      pruneLeastImportant_History         (iHistory *);
@@ -78,7 +78,7 @@ size_t      pruneLeastImportantMemory_History   (iHistory *);
 void        invalidateTheme_History             (iHistory *); /* theme has changed, cached contents need updating */
 void        invalidateCachedLayout_History      (iHistory *);
 
-iBool       atLatest_History            (const iHistory *);
+iBool       atNewest_History            (const iHistory *);
 iBool       atOldest_History            (const iHistory *);
 
 const iStringArray *   searchContents_History   (const iHistory *, const iRegExp *pattern); /* chronologically ascending */
