@@ -123,3 +123,6 @@ struct Impl_MediaRequest {
 
 iDeclareObjectConstructionArgs(MediaRequest, iDocumentWidget *doc, unsigned int linkId,
                                const iString *url, iBool enableFilters)
+    
+iMediaRequest * newReused_MediaRequest  (iDocumentWidget *doc, unsigned int linkId,
+                                         iGmRequest *request);
