@@ -196,6 +196,14 @@ static void keyStr_LabelWidget_(const iLabelWidget *d, iString *str) {
     toString_Sym(d->key, d->kmods, str);
 }
 
+static iBool areTabButtonsThemeColored_(void) {
+    const enum iGmDocumentTheme docTheme = docTheme_Prefs(prefs_App());
+    const iBool isDarkUI = isDark_ColorTheme(colorTheme_App());
+    return (docTheme == colorfulLight_GmDocumentTheme ||
+            docTheme == sepia_GmDocumentTheme ||
+            (docTheme == oceanic_GmDocumentTheme && !isDarkUI));
+}
+
 static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int *frame1, int *frame2,
                                    int *icon, int *meta) {
     const iWidget *w           = constAs_Widget(d);
@@ -234,13 +242,19 @@ static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int
             }
             else {
                 const enum iGmDocumentTheme docTheme = docTheme_Prefs(prefs_App());
-                if ((docTheme == colorfulLight_GmDocumentTheme || docTheme == sepia_GmDocumentTheme) &&
+                if (areTabButtonsThemeColored_() &&
                     !cmp_String(&d->widget.parent->id, "tabs.buttons")) {
-                    *bg = (docTheme == sepia_GmDocumentTheme &&
-                                   colorTheme_App() == pureWhite_ColorTheme
+                    *bg = (docTheme == oceanic_GmDocumentTheme ||
+                                   (docTheme == sepia_GmDocumentTheme &&
+                                    colorTheme_App() == pureWhite_ColorTheme)
                                ? tmBackground_ColorId
                                : tmBannerBackground_ColorId);
                     isThemeBackground = iTrue;
+                    /* Ensure visibility in case the background matches UI background. */
+                    if (delta_Color(get_Color(*bg), get_Color(uiBackground_ColorId)) < 30) {
+                        *bg = uiBackgroundSelected_ColorId;
+                        isThemeBackground = iFalse;
+                    }
                 }
                 else {
                     *bg = uiBackgroundSelected_ColorId;
