@@ -81,6 +81,20 @@ static enum iFontId labelBoldFont_(void) {
     return deviceType_App() == phone_AppDeviceType ? uiLabelBigBold_FontId : uiLabelMediumBold_FontId;
 }
 
+iBool isFullSizePanel_Mobile(const iWidget *panels) {
+    /* TODO: Set panel type as a creation parameter. */
+    const char *id = cstr_String(id_Widget(panels));
+    if (deviceType_App() == tablet_AppDeviceType) {
+        return !iCmpStr(id, "prefs") || !iCmpStr(id, "upload");
+    }
+    return !iCmpStr(id, "prefs") || startsWith_CStr(id, "bmed") || startsWith_CStr(id, "sitespec ")
+        || !iCmpStr(id, "upload") || !iCmpStr(id, "certimport") || !iCmpStr(id, "ident");
+}
+
+iLocalDef iBool isFullSizePanel_(const iWidget *panels) {
+    return isFullSizePanel_Mobile(panels);
+}
+
 static void updatePanelSheetMetrics_(iWidget *sheet) {
     iWidget *navi       = findChild_Widget(sheet, "panel.navi");
     int      naviHeight = lineHeight_Text(labelFont_()) + 4 * gap_UI;
@@ -89,8 +103,23 @@ static void updatePanelSheetMetrics_(iWidget *sheet) {
 #if defined (iPlatformAppleMobile)
     safeAreaInsets_iOS(&left, &top, &right, &bottom);
 #endif
-    setPadding_Widget(sheet, left, 0, right, 0);
-    navi->rect.pos = init_I2(left, top);
+    if (isFullSizePanel_(sheet)) {
+        setPadding_Widget(sheet, left, 0, right, 0);
+        navi->rect.pos = init_I2(left, top);
+    }
+    else {
+        setPadding_Widget(sheet, 0, top, 0, bottom);
+//        if (deviceType_App() == tablet_AppDeviceType) {
+//            const iRect visRect = visibleRect_Root(sheet->root);
+//            sheet->rect.pos.y = windowToLocal_Widget(sheet,
+//                                                     init_I2(0, iMaxi(top_Rect(visRect), bottom_Rect(visRect) -
+//                                                             sheet->rect.size.y))).y;
+//            const int excess = bottom_Rect(bounds_Widget(sheet)) - bottom_Rect(visRect);
+//            if (excess > 0) {
+//                sheet->rect.size.y -= excess;
+//            }
+//        }
+    }
     iConstForEach(PtrArray, i, findChildren_Widget(sheet, "panel.toppad")) {
         iWidget *pad = *i.value;
         setFixedSize_Widget(pad, init1_I2(naviHeight));
@@ -136,16 +165,20 @@ static void updateCertListHeight_(iWidget *detailStack) {
 }
 
 static iBool mainDetailSplitHandler_(iWidget *mainDetailSplit, const char *cmd) {
+    iWidget *sheet = parent_Widget(mainDetailSplit);
     if (equal_Command(cmd, "window.resized")) {
-        const iBool  isPortrait   = (deviceType_App() == phone_AppDeviceType && isPortrait_App());
-        const iRect  safeRoot     = safeRect_Root(mainDetailSplit->root);
-        iWidget *    sheet        = parent_Widget(mainDetailSplit);
-        iWidget *    navi         = findChild_Widget(sheet, "panel.navi");
-        iWidget *    detailStack  = findChild_Widget(mainDetailSplit, "detailstack");
-        const size_t numPanels    = childCount_Widget(detailStack);
-        const iBool  isSideBySide = isSideBySideLayout_() && numPanels > 0;
-        setPos_Widget(mainDetailSplit, topLeft_Rect(safeRoot));
-        setFixedSize_Widget(mainDetailSplit, safeRoot.size);
+        const iBool   isPortrait   = (deviceType_App() == phone_AppDeviceType && isPortrait_App());
+        const iRect   safeRoot     = safeRect_Root(mainDetailSplit->root);
+        const iBool   isFullSize   = isFullSizePanel_(sheet);
+        iWidget      *navi         = findChild_Widget(sheet, "panel.navi");
+        iLabelWidget *naviTitle    = findChild_Widget(navi, "navi.title");
+        iWidget      *detailStack  = findChild_Widget(mainDetailSplit, "detailstack");
+        const size_t  numPanels    = childCount_Widget(detailStack);
+        const iBool   isSideBySide = isSideBySideLayout_() && numPanels > 0;
+        if (isFullSize) {
+            setPos_Widget(mainDetailSplit, topLeft_Rect(safeRoot));
+            setFixedSize_Widget(mainDetailSplit, safeRoot.size);
+        }
         setFlags_Widget(mainDetailSplit, arrangeHorizontal_WidgetFlag, isSideBySide);
         setFlags_Widget(detailStack, expand_WidgetFlag, isSideBySide);
         setFlags_Widget(detailStack, hidden_WidgetFlag, numPanels == 0);
@@ -157,27 +190,13 @@ static iBool mainDetailSplitHandler_(iWidget *mainDetailSplit, const char *cmd)
                                          (deviceType_App() == phone_AppDeviceType ?
                                           safeRoot.size.x * 2 / 5 : safeRoot.size.x / 3));
         }
+        setTextOffset_LabelWidget(
+            naviTitle, init_I2(isFullSize && isSideBySide ? topPanel->rect.size.x / 2 : 0, 0));
         if (deviceType_App() == tablet_AppDeviceType) {
             setPadding_Widget(topPanel, pad, 0, pad, pad);
-#if 0
-            if (numPanels == 0) {
-                setFlags_Widget(sheet, centerHorizontal_WidgetFlag, iTrue);
-                const int sheetWidth = iMin(safeRoot.size.x, safeRoot.size.y);
-                mainDetailSplit->rect.size.x = sheetWidth;
-                setFixedSize_Widget(sheet, init_I2(sheetWidth, -1));
-                setFixedSize_Widget(navi, init_I2(sheetWidth, -1));
-            }
-#endif
-        }
-        iWidget *detailTitle = findChild_Widget(navi, "detailtitle"); {
-            setPos_Widget(detailTitle, init_I2(width_Widget(topPanel), 0));
-            setFixedSize_Widget(detailTitle,
-                                init_I2(width_Widget(detailStack), height_Widget(navi)));
-            setFlags_Widget(detailTitle, hidden_WidgetFlag, !isSideBySide);
         }
         iForEach(ObjectList, i, children_Widget(detailStack)) {
             iWidget *panel = i.object;
-            setFlags_Widget(findTitleLabel_(panel), hidden_WidgetFlag, isSideBySide);
             setFlags_Widget(panel, leftEdgeDraggable_WidgetFlag, !isSideBySide);
             if (isSideBySide) {
                 setVisualOffset_Widget(panel, 0, 0, 0);
@@ -187,6 +206,14 @@ static iBool mainDetailSplitHandler_(iWidget *mainDetailSplit, const char *cmd)
         arrange_Widget(mainDetailSplit);
         updateCertListHeight_(detailStack);
     }
+    else if (deviceType_App() == tablet_AppDeviceType && equal_Command(cmd, "keyboard.changed")) {
+        if (arg_Command(cmd) > 0 && !isFullSizePanel_(sheet)) {
+            /* Softare keyboard shown. */
+            animateToRootVisibleTop_Widget(sheet, 300);
+            postCommand_Root(sheet->root, "input.overflow");
+        }
+        return iFalse;
+    }
     else if (equal_Command(cmd, "mouse.clicked") && arg_Command(cmd)) {
         if (focus_Widget() && class_Widget(focus_Widget()) == &Class_InputWidget) {
             setFocus_Widget(NULL);
@@ -212,11 +239,24 @@ iWidget *panel_Mobile(const iWidget *panels, size_t index) {
     return child_Widget(findChild_Widget(panels, "detailstack"), index);
 }
 
+static void updateNaviActionVisibility_(iWidget *sheet, iWidget *curPanel) {
+    iWidget      *navi        = findChild_Widget(sheet, "panel.navi");
+    iWidget      *naviActions = findChild_Widget(navi, "navi.actions");
+    iForEach(ObjectList, i, children_Widget(naviActions)) {
+        setFlags_Widget(i.object, hidden_WidgetFlag,
+                        userData_Object(i.object) && userData_Object(i.object) != curPanel);
+    }
+    arrange_Widget(navi);
+    refresh_Widget(navi);    
+}
+
 static iBool topPanelHandler_(iWidget *topPanel, const char *cmd) {
     const iBool isPortrait = !isSideBySideLayout_();
+    // sheet > mdsplit > panel.top
+    iWidget *sheet = parent_Widget(parent_Widget(topPanel));
     if (equal_Command(cmd, "panel.open")) {
         /* This command is sent by the button that opens the panel. */
-        iWidget *button = pointer_Command(cmd);        
+        iWidget *button = pointer_Command(cmd);
         iWidget *panel = userData_Object(button);
         unselectAllPanelButtons_(topPanel);
         int panelIndex = -1;
@@ -231,12 +271,11 @@ static iBool topPanelHandler_(iWidget *topPanel, const char *cmd) {
             }
             childIndex++;
         }
-        iLabelWidget *detailTitle =
-            findChild_Widget(parent_Widget(parent_Widget(topPanel)), "detailtitle");
-//        setFlags_Widget(as_Widget(detailTitle), hidden_WidgetFlag, !isSideBySideLayout_());
-        setFont_LabelWidget(detailTitle, uiLabelLargeBold_FontId);
-        setTextColor_LabelWidget(detailTitle, uiHeading_ColorId);
-        setText_LabelWidget(detailTitle, text_LabelWidget((iLabelWidget *) findTitleLabel_(panel)));
+        /* Update the navigation bar. */ {
+            iLabelWidget *naviTitle = findChild_Widget(sheet, "navi.title");
+            updateText_LabelWidget(naviTitle, text_LabelWidget((iLabelWidget *) findTitleLabel_(panel)));
+            updateNaviActionVisibility_(sheet, panel);
+        }        
         setFlags_Widget(button, selected_WidgetFlag, iTrue);
         postCommand_Widget(topPanel, "panel.changed arg:%d", panelIndex);
         updateCertListHeight_(findDetailStack_(topPanel));
@@ -256,16 +295,25 @@ static iBool topPanelHandler_(iWidget *topPanel, const char *cmd) {
                     setupSheetTransition_Mobile(child, iFalse);
                     setFlags_Widget(child, hidden_WidgetFlag | disabled_WidgetFlag, iTrue);
                     setFocus_Widget(NULL);
-                    updateTextCStr_LabelWidget(findWidget_App("panel.back"), "Back");
+                    setTextCStr_LabelWidget(findWidget_App("panel.back"), "Back");
+                    updateTextCStr_LabelWidget(findWidget_App("navi.title"), "");
                     wasClosed = iTrue;
                     postCommand_Widget(topPanel, "panel.changed arg:-1");
                 }
             }
         }
+        updateNaviActionVisibility_(sheet, topPanel);
         unselectAllPanelButtons_(topPanel);
         if (!wasClosed) {
+            iWidget *widget = NULL;
             /* TODO: Should come up with a more general-purpose approach here. */
-            if (findWidget_App("prefs")) {
+            if (findWidget_App("ident")) {
+                postCommand_App("ident.cancel");
+            }
+            else if ((widget = findWidget_App("certimport")) != NULL) {
+                postCommand_Widget(widget, "cancel");
+            }
+            else if (findWidget_App("prefs")) {
                 postCommand_App("prefs.dismiss");
             }
             else if (findWidget_App("upload")) {
@@ -287,13 +335,21 @@ static iBool topPanelHandler_(iWidget *topPanel, const char *cmd) {
         }
         return iTrue;
     }
-    if (equal_Command(cmd, "document.changed")) {
+    else if (equal_Command(cmd, "document.changed")) {
         postCommand_App("prefs.dismiss");
         return iFalse;
     }
-    if (equal_Command(cmd, "window.resized")) {
-        // sheet > mdsplit > panel.top
-        updatePanelSheetMetrics_(parent_Widget(parent_Widget(topPanel)));
+    else if (equal_Command(cmd, "window.resized") || equal_Command(cmd, "keyboard.changed")) {
+        updatePanelSheetMetrics_(sheet);
+    }
+    else if (equalWidget_Command(cmd, sheet, "input.resized")) {
+        const int rev = arg_Command(cmd);
+        if (sheet->root->pendingArrange < rev) {
+            sheet->root->pendingArrange = rev;
+            arrange_Widget(sheet);
+            refresh_Widget(pointer_Command(cmd)); /* may be on a buffered panel */
+        }
+        return iTrue;
     }
     return iFalse;
 }
@@ -430,7 +486,6 @@ static iWidget *makeValuePaddingWithHeading_(iLabelWidget *heading, iWidget *val
     setPadding_Widget(div, gap_UI, gap_UI, 4 * gap_UI, gap_UI);
     addChildFlags_Widget(div, iClob(heading), 0);
     setPadding1_Widget(as_Widget(heading), 0);
-    //setFixedSize_Widget(as_Widget(heading), init_I2(-1, height_Widget(value)));
     setFont_LabelWidget(heading, labelFont_());
     setTextColor_LabelWidget(heading, uiTextStrong_ColorId);
     if (isInput && ~value->flags & fixedWidth_WidgetFlag) {
@@ -479,12 +534,7 @@ static iWidget *addChildPanel_(iWidget *parent, iLabelWidget *panelButton,
     return panel;
 }
 
-//void finalizeSheet_Mobile(iWidget *sheet) {
-//    arrange_Widget(sheet);
-//    postRefresh_App();
-//}
-
-static size_t countItems_(const iMenuItem *itemsNullTerminated) {
+size_t count_MenuItem(const iMenuItem *itemsNullTerminated) {
     size_t num = 0;
     for (; itemsNullTerminated->label; num++, itemsNullTerminated++) {}
     return num;
@@ -523,14 +573,25 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
     if (hasLabel_Command(spec, "device") && deviceType_App() != argLabel_Command(spec, "device")) {
         return;
     }
+    if (hasLabel_Command(spec, "android")) {
+        const int requireAndroid = argLabel_Command(spec, "android");
+#if defined (iPlatformAndroid)
+        if (!requireAndroid) return;
+#else
+        if (requireAndroid) return;
+#endif
+    }
     if (equal_Command(spec, "title")) {
-        iLabelWidget *title = addChildFlags_Widget(panel,
-                                                   iClob(new_LabelWidget(label, NULL)),
-                                                   alignLeft_WidgetFlag | frameless_WidgetFlag |
-                                                   collapse_WidgetFlag);
+        iLabelWidget *title =
+            addChildFlags_Widget(panel,
+                                 iClob(new_LabelWidget(label, NULL)),
+                                 alignLeft_WidgetFlag | frameless_WidgetFlag | collapse_WidgetFlag);
+        if (cmp_String(id_Widget(panel), "panel.top")) {
+            /* Child panel titles are shown in the navi bar. */
+            setFlags_Widget(as_Widget(title), hidden_WidgetFlag, iTrue);
+        }
         setFont_LabelWidget(title, uiLabelLargeBold_FontId);
         setTextColor_LabelWidget(title, uiHeading_ColorId);
-//        setAllCaps_LabelWidget(title, iTrue);
         setId_Widget(as_Widget(title), id);
     }
     else if (equal_Command(spec, "heading")) {
@@ -550,7 +611,7 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
     else if (equal_Command(spec, "dropdown")) {
         const iMenuItem *dropItems = item->data;
         iLabelWidget *drop = makeMenuButton_LabelWidget(dropItems[0].label,
-                                                        dropItems, countItems_(dropItems));
+                                                        dropItems, count_MenuItem(dropItems));
         value = as_Widget(drop);
         setFont_LabelWidget(drop, labelFont_());
         setFlags_Widget(as_Widget(drop),
@@ -565,25 +626,36 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
     else if (equal_Command(spec, "radio") || equal_Command(spec, "buttons")) {
         const iBool isRadio = equal_Command(spec, "radio");
         const iBool isHorizontal = argLabel_Command(spec, "horizontal");
+        const int rowLen = argLabel_Command(spec, "rowlen");
         addChild_Widget(panel, iClob(makePadding_Widget(lineHeight_Text(labelFont_()))));
         iLabelWidget *head = makeHeading_Widget(label);
         setAllCaps_LabelWidget(head, iTrue);
         setRemoveTrailingColon_LabelWidget(head, iTrue);
         addChild_Widget(panel, iClob(head));
         widget = new_Widget();
+        iWidget *subDiv = widget;
         setBackgroundColor_Widget(widget, uiBackgroundSidebar_ColorId);
         const int hPad = (isHorizontal ? 0 : 1);
         setPadding_Widget(widget, hPad * gap_UI, 2 * gap_UI, hPad * gap_UI, 2 * gap_UI);
         setFlags_Widget(widget,
                         borderTop_WidgetFlag |
                             borderBottom_WidgetFlag |
-                            (isHorizontal ? arrangeHorizontal_WidgetFlag : arrangeVertical_WidgetFlag) |
+                            (isHorizontal && !rowLen ? arrangeHorizontal_WidgetFlag : arrangeVertical_WidgetFlag) |
                             arrangeHeight_WidgetFlag |
                             resizeToParentWidth_WidgetFlag |
                             resizeWidthOfChildren_WidgetFlag,
                         iTrue);
+        if (rowLen) {
+            subDiv = new_Widget();
+            addChildFlags_Widget(widget, iClob(subDiv),
+                                 arrangeHorizontal_WidgetFlag |
+                                 arrangeHeight_WidgetFlag |
+                                 resizeToParentWidth_WidgetFlag |
+                                 resizeWidthOfChildren_WidgetFlag);
+        }
         setId_Widget(widget, id);
         iBool isFirst = iTrue;
+        int numCols = 0;
         for (const iMenuItem *radioItem = item->data; radioItem->label; radioItem++) {
             if (!isHorizontal && !isFirst) {
                 /* The separator is padded from the left so we need two. */
@@ -598,7 +670,7 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
             }
             isFirst = iFalse;
             const char *  radId = cstr_Command(radioItem->label, "id");
-            int64_t       flags = noBackground_WidgetFlag | frameless_WidgetFlag;
+            int64_t       flags = noBackground_WidgetFlag| frameless_WidgetFlag;
             if (!isHorizontal) {
                 flags |= alignLeft_WidgetFlag;
             }
@@ -624,7 +696,16 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
             setPadding_Widget(as_Widget(button), gap_UI, 1 * gap_UI, 0, 1 * gap_UI);
             updateSize_LabelWidget(button);
             setPadding_Widget(widget, 0, 0, 0, 0);
-            addChildFlags_Widget(widget, iClob(button), flags);
+            addChildFlags_Widget(subDiv, iClob(button), flags);
+            if (rowLen && ++numCols == rowLen) {
+                numCols = 0;
+                subDiv = new_Widget();
+                addChildFlags_Widget(widget, iClob(subDiv),
+                                     arrangeHorizontal_WidgetFlag |
+                                     arrangeHeight_WidgetFlag |
+                                     resizeToParentWidth_WidgetFlag |
+                                     resizeWidthOfChildren_WidgetFlag);
+            }
         }
     }
     else if (equal_Command(spec, "input")) {
@@ -663,6 +744,7 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
         iListWidget *list = (iListWidget *) certList;
         setBackgroundColor_Widget(as_Widget(list), uiBackgroundSidebar_ColorId);
         widget = as_Widget(certList);
+        setFlags_Widget(widget, borderTop_WidgetFlag | borderBottom_WidgetFlag, iTrue);
         updateItems_CertListWidget(certList);
         invalidate_ListWidget(list);
     }
@@ -670,6 +752,21 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
         widget = as_Widget(heading = makePanelButton_(label, item->command));
         setFlags_Widget(widget, selected_WidgetFlag, argLabel_Command(spec, "selected") != 0);
     }
+    else if (equal_Command(spec, "navi.action")) {
+        iLabelWidget *action = new_LabelWidget(label, item->command);
+        setId_Widget(as_Widget(action), id);
+        setFlags_Widget(as_Widget(action),
+                        hidden_WidgetFlag | collapse_WidgetFlag | frameless_WidgetFlag |
+                            noBackground_WidgetFlag | extraPadding_WidgetFlag,
+                        iTrue);
+        setFont_LabelWidget(action, labelBoldFont_());
+        setTextColor_LabelWidget(action, uiTextAction_ColorId);
+        setUserData_Object(action, panel); /* visible on this panel */
+        iWidget *naviActions = findChild_Widget(parent_Widget(findParent_Widget(panel, "mdsplit")),
+                                                "navi.actions");
+        iAssert(naviActions);
+        addChild_Widget(naviActions, iClob(action));
+    }
     else if (equal_Command(spec, "label")) {
         iLabelWidget *lab = new_LabelWidget(label, NULL);
         widget = as_Widget(lab);
@@ -689,6 +786,7 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
             height *= argfLabel_Command(spec, "arg");
         }
         widget = makePadding_Widget(lineHeight_Text(labelFont_()) * height);
+//        setBackgroundColor_Widget(widget, hasLabel_Command(spec, "arg") ? green_ColorId : red_ColorId);
     }
     /* Apply common styling to the heading. */
     if (heading) {
@@ -743,25 +841,49 @@ iWidget *makePanelsParent_Mobile(iWidget *parentWidget,
     return panels;
 }
 
-void initPanels_Mobile(iWidget *panels, iWidget *parentWidget, 
+void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
                        const iMenuItem *itemsNullTerminated,
                        const iMenuItem *actions, size_t numActions) {
     /* A multipanel widget has a top panel and one or more detail panels. In a horizontal layout,
        the detail panels slide in from the right and cover the top panel. In a landscape layout,
        the detail panels are always visible on the side. */
     setBackgroundColor_Widget(panels, uiBackground_ColorId);
+    const iBool isFullHeight = isFullSizePanel_(panels);
     setFlags_Widget(panels,
-                    resizeToParentWidth_WidgetFlag | resizeToParentHeight_WidgetFlag |
-                        frameless_WidgetFlag | focusRoot_WidgetFlag | commandOnClick_WidgetFlag |
-                        horizontalOffset_WidgetFlag |
-                        /*overflowScrollable_WidgetFlag |*/ leftEdgeDraggable_WidgetFlag,
+                    resizeToParentWidth_WidgetFlag |
+                        (isFullHeight
+                             ? resizeToParentHeight_WidgetFlag | leftEdgeDraggable_WidgetFlag |
+                                   horizontalOffset_WidgetFlag
+                             : (arrangeHeight_WidgetFlag | moveToParentBottomEdge_WidgetFlag)) |
+                        frameless_WidgetFlag | focusRoot_WidgetFlag | commandOnClick_WidgetFlag,
                     iTrue);
+    if (!isFullHeight) {
+        panels->minSize.y = 60 * gap_UI;
+        if (deviceType_App() == tablet_AppDeviceType) {
+            setFlags_Widget(panels, resizeToParentWidth_WidgetFlag, iFalse);
+            setFlags_Widget(panels, centerHorizontal_WidgetFlag, iTrue);
+            const iRect safe = safeRect_Root(panels->root);
+            setFixedSize_Widget(panels, init_I2(iMin(safe.size.x, safe.size.y), -1));
+            //panels->maxHeight = safe.size.y;
+        }
+        else if (isLandscapePhone_App()) {
+            setFlags_Widget(panels, resizeToParentWidth_WidgetFlag, iFalse);
+            setFlags_Widget(panels, centerHorizontal_WidgetFlag, iTrue);
+            const iRect safe = safeRect_Root(panels->root);
+            setFixedSize_Widget(panels, init_I2(safe.size.x * 0.8, -1));
+            //panels->maxHeight = safe.size.y;
+        }
+    }
     panels->flags2 |= fadeBackground_WidgetFlag2;
     setFlags_Widget(panels, overflowScrollable_WidgetFlag, iFalse);
     /* The top-level split between main and detail panels. */
     iWidget *mainDetailSplit = makeHDiv_Widget(); {
         setCommandHandler_Widget(mainDetailSplit, mainDetailSplitHandler_);
         setFlags_Widget(mainDetailSplit, resizeHeightOfChildren_WidgetFlag, iFalse);
+        if (!isFullHeight) {
+            setFlags_Widget(
+                mainDetailSplit, resizeToParentWidth_WidgetFlag | arrangeHeight_WidgetFlag, iTrue);
+        }
         setId_Widget(mainDetailSplit, "mdsplit");
         addChild_Widget(panels, iClob(mainDetailSplit));
     }
@@ -778,6 +900,11 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
         addChild_Widget(mainDetailSplit, iClob(topPanel));
         setId_Widget(addChild_Widget(topPanel, iClob(makePadding_Widget(0))), "panel.toppad");
     }
+    if (!isFullHeight) {
+        /* Scroll the entire dialog. */
+        setFlags_Widget(topPanel, overflowScrollable_WidgetFlag, iFalse);
+        setFlags_Widget(panels, overflowScrollable_WidgetFlag | drawBackgroundToBottom_WidgetFlag, iTrue);
+    }
     iWidget *detailStack = new_Widget(); {
         setId_Widget(detailStack, "detailstack");
         setFlags_Widget(detailStack, collapse_WidgetFlag | resizeWidthOfChildren_WidgetFlag, iTrue);
@@ -789,28 +916,41 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
     }
     /* Navigation bar at the top. */
     iLabelWidget *naviBack;
+    iWidget *naviActions;
     iWidget *navi = new_Widget(); {
         setId_Widget(navi, "panel.navi");
         setBackgroundColor_Widget(navi, uiBackground_ColorId);
-        setId_Widget(addChildFlags_Widget(navi,
-                                          iClob(new_LabelWidget("", NULL)),
-                                          alignLeft_WidgetFlag | fixedPosition_WidgetFlag |
-                                              fixedSize_WidgetFlag | hidden_WidgetFlag |
-                                              frameless_WidgetFlag),
-                     "detailtitle");
+        iLabelWidget *naviTitle = addChildFlags_Widget(
+            navi,
+            iClob(new_LabelWidget("", NULL)),
+            noBackground_WidgetFlag | frameless_WidgetFlag | resizeToParentWidth_WidgetFlag |
+                resizeToParentHeight_WidgetFlag | moveToParentLeftEdge_WidgetFlag);
+        setId_Widget(as_Widget(naviTitle), "navi.title");
+        setFont_LabelWidget(naviTitle, labelFont_());
+        //setTextColor_LabelWidget(naviTitle, uiTextStrong_ColorId);
         naviBack = addChildFlags_Widget(
             navi,
             iClob(newKeyMods_LabelWidget(
-                leftAngle_Icon " ${panel.back}", SDLK_ESCAPE, 0, "panel.close")),
+                isFullHeight ? leftAngle_Icon " ${panel.back}" : "${close}",
+                SDLK_ESCAPE, 0, "panel.close")),
             noBackground_WidgetFlag | frameless_WidgetFlag | alignLeft_WidgetFlag |
                 extraPadding_WidgetFlag);
         checkIcon_LabelWidget(naviBack);
         setId_Widget(as_Widget(naviBack), "panel.back");
         setFont_LabelWidget(naviBack, labelFont_());
+        //if (!isFullHeight) {
+            setTextColor_LabelWidget(naviBack, uiTextAction_ColorId);
+        //}
+        naviActions = addChildFlags_Widget(
+            navi,
+            iClob(new_Widget()),
+            noBackground_WidgetFlag | frameless_WidgetFlag | moveToParentRightEdge_WidgetFlag |
+                arrangeSize_WidgetFlag | arrangeHorizontal_WidgetFlag);
+        setId_Widget(naviActions, "navi.actions");
         addChildFlags_Widget(panels, iClob(navi),
-                             drawBackgroundToVerticalSafeArea_WidgetFlag |
-                                 arrangeHeight_WidgetFlag | resizeWidthOfChildren_WidgetFlag |
-                                 resizeToParentWidth_WidgetFlag | arrangeVertical_WidgetFlag);        
+                             (isFullHeight ? drawBackgroundToVerticalSafeArea_WidgetFlag : 0) |
+                                 arrangeHeight_WidgetFlag | //resizeWidthOfChildren_WidgetFlag |
+                                 resizeToParentWidth_WidgetFlag | arrangeVertical_WidgetFlag);    
     }
     iBool haveDetailPanels = iFalse;
     /* Create panel contents based on provided items. */
@@ -848,7 +988,7 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
         const iMenuItem *defaultItem = &actions[numActions - 1];
         iAssert(defaultItem);
         if (defaultItem && !cancelItem) {
-            updateTextCStr_LabelWidget(naviBack, defaultItem->label);
+            setTextCStr_LabelWidget(naviBack, defaultItem->label);
             setCommand_LabelWidget(naviBack, collectNewCStr_String(defaultItem->command));
             setFlags_Widget(as_Widget(naviBack), alignLeft_WidgetFlag, iFalse);
             setFlags_Widget(as_Widget(naviBack), alignRight_WidgetFlag, iTrue);
@@ -857,7 +997,7 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
         }
         else if (defaultItem && defaultItem != cancelItem) {
             if (!haveDetailPanels) {
-                updateTextCStr_LabelWidget(naviBack, cancelItem->label);
+                setTextCStr_LabelWidget(naviBack, cancelItem->label);
                 setCommand_LabelWidget(naviBack, collectNewCStr_String(cancelItem->command
                                                                        ? cancelItem->command
                                                                        : "cancel"));
@@ -868,8 +1008,7 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
                             frameless_WidgetFlag | extraPadding_WidgetFlag |
                                 noBackground_WidgetFlag,
                             iTrue);
-            addChildFlags_Widget(as_Widget(naviBack), iClob(defaultButton),
-                                 moveToParentRightEdge_WidgetFlag);
+            addChildFlags_Widget(naviActions, iClob(defaultButton), 0);
             updateSize_LabelWidget(defaultButton);
         }
         /* All other actions are added as buttons. */
@@ -905,7 +1044,9 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
     mainDetailSplitHandler_(mainDetailSplit, "window.resized"); /* make it resize the split */
     updatePanelSheetMetrics_(panels);
     arrange_Widget(panels);
-    postCommand_App("widget.overflow"); /* with the correct dimensions */    
+    if (!isFullHeight) {
+        arrange_Widget(panels);
+    }
 //    printTree_Widget(panels);
 }
 
@@ -940,7 +1081,7 @@ void setupMenuTransition_Mobile(iWidget *sheet, iBool isIncoming) {
     }
     const int maxOffset = isHorizPanel            ? width_Widget(sheet)
                           : isPortraitPhone_App() ? height_Widget(sheet)
-                                                  : (6 * gap_UI);
+                                                  : (4 * gap_UI);
     if (isIncoming) {
         setVisualOffset_Widget(sheet, maxOffset, 0, 0);
         setVisualOffset_Widget(sheet, 0, 330, easeOut_AnimFlag | softer_AnimFlag);
@@ -970,10 +1111,6 @@ void setupSheetTransition_Mobile(iWidget *sheet, int flags) {
         }
         return;
     }
-    if (isSideBySideLayout_()) {
-        /* TODO: Landscape transitions? */
-        return;
-    }
     setFlags_Widget(sheet,
                     horizontalOffset_WidgetFlag,
                     dir == right_TransitionDir || dir == left_TransitionDir);
@@ -993,7 +1130,8 @@ void setupSheetTransition_Mobile(iWidget *sheet, int flags) {
                 setVisualOffset_Widget(sheet, height_Widget(sheet), 0, 0);
                 break;
         }
-        setVisualOffset_Widget(sheet, 0, 200, easeOut_AnimFlag);
+        setVisualOffset_Widget(sheet, 0, deviceType_App() == tablet_AppDeviceType ? 350 : 275,
+                               easeOut_AnimFlag | softer_AnimFlag);
     }        
     else {
         switch (dir) {
@@ -1019,6 +1157,26 @@ void setupSheetTransition_Mobile(iWidget *sheet, int flags) {
     }
 }
 
+int leftSafeInset_Mobile(void) {
+#if defined (iPlatformAppleMobile)
+    float left;
+    safeAreaInsets_iOS(&left, NULL, NULL, NULL);
+    return iRound(left);
+#else
+    return 0;
+#endif
+}
+
+int topSafeInset_Mobile(void) {
+#if defined (iPlatformAppleMobile)
+    float top;
+    safeAreaInsets_iOS(NULL, &top, NULL, NULL);
+    return iRound(top);
+#else
+    return 0;
+#endif
+}
+
 int bottomSafeInset_Mobile(void) {
 #if defined (iPlatformAppleMobile)
     float bot;
